#!/bin/sh
#################################################################
# nom			: install.sh
# description	: Scrit d'installation de l'agent DSR sous systèmes de type unix
# version		: 1.10
# date			: 17.12.2013 15:03:04
# autheur		: remi
#################################################################
# installe correctement sous ESXI (pas de X64 ni de install)
# LISTE DES CODES ERREUR POUR install.sh
#  1 "Echec de création du chemin /usr/local"
#  2 "Echec de création du chemin /usr/local/dsr"
#  3 "Dossier de scripts non détecté"
#  4 "Echec de création du chemin /usr/local/dsr/scripts"
#  5 "Echec d'installation du fichier /usr/local/dsr/scripts/backup_config.sh"
#  6 "Echec d'ajout du droit d'exécution au fichier /usr/local/dsr/scripts/backup_config.sh"
#  7 "Impossible d'arrêter l'ancienne version de DSR."
#  8 "Echec d'installation du fichier $init_dir/dsr"
#  9 "Echec d'ajout du droit d'exécution au fichier $init_dir/dsr"
# 10 "\nLes Scripts de démarrage n'ont pas été créé automatiquement\nPour que Data Safe Restore se lance au démarrage de votre machine\najoutez $init_dir/dsr start a vos scripts de démarrage"
# 11 "Echec lors de l'exécution /usr/local/dsr/scripts/backup_config.sh"
# 12 "Installation Data Safe Restore Client - OK, mais impossible de démarrer l'agent"
# 13 "Echec d'installation du fichier /usr/local/dsr/dsr"
# 99 "Vous devez être root pour lancer ce script."
# ****************************************
# rollbackInstall
# Descritpion :
#	Cette fonction annule les opérations effectuées jusqu'au point d'une erreur dans l'installation en se basant
#	sur le paramètre de niveau de rollback
# Paramètres :
#	1 : est le niveau de rollback à effectuer. Cette variable est une chaine. A chaque opération correspond à un caractère.
#	    Si ce caractère est présent dans la chaine c'est que cette opération a été effectuée et doit être annulée.
#	2 : le chemin init_dir
# ****************************************
rollbackInstall ()
{
	if [ "$1" ]; then
		echo "Annulation de l'installation... "
	fi
	# enlève le script de démarrage DSR des scripts à lancer automatiquement (update-rc.d)
	if [ `echo "$1" | grep j` ]; then
		/usr/sbin/update-rc.d -f dsr remove
	fi
	# enlève le script de démarrage DSR des scripts à lancer automatiquement (rc-update)
	if [ `echo "$1" | grep i` ]; then
		/sbin/rc-update delete dsr default
	fi
	# enlève le script de démarrage DSR des scripts à lancer automatiquement (chkconfig)
	if [ `echo "$1" | grep h` ]; then
		chkconfig --del dsr
	fi
	# supprime $init_dir/dsr
	if [ `echo "$1" | grep g` ]; then
		if [ -e $2/dsr.beemoinstallbak ]; then
			mv $2/dsr.beemoinstallbak $2/dsr
		else
			rm $2/dsr
		fi
	fi
	if [ `echo "$1" | grep f` ]; then
		$2/dsr start
	fi
	# supprime /usr/local/dsr/scripts/backup_config.sh  $init_dir/dsr.beemoinstallbak
	if [ `echo "$1" | grep e` ]; then
		if [ -e /usr/local/dsr/scripts/backup_config.sh.beemoinstallbak ]; then
			mv /usr/local/dsr/scripts/backup_config.sh.beemoinstallbak /usr/local/dsr/scripts/backup_config.sh
		else
			rm /usr/local/dsr/scripts/backup_config.sh
		fi
	fi
	# supprime /usr/local/dsr/scripts
	if [ `echo "$1" | grep d` ]; then
		rmdir /usr/local/dsr/scripts
	fi
	# supprime /usr/local/dsr/dsr
	if [ `echo "$1" | grep c` ]; then
		if [ -e /usr/local/dsr/dsr.beemoinstallbak ]; then
			mv  /usr/local/dsr/dsr.beemoinstallbak /usr/local/dsr/dsr
		else
			rm /usr/local/dsr/dsr
		fi
	fi
	# supprime /usr/local/dsr
	if [ `echo "$1" | grep b` ]; then
		rmdir /usr/local/dsr
	fi
	# supprime /usr/local
	if [ `echo "$1" | grep a` ]; then
		rmdir /usr/local
	fi
	if [ "$1" ]; then
		echo "Terminé"
	fi
}
# ****************************************
# handleError
# Descritpion :
#	Cette fonction sert à gérer le cas d'une erreur ou d'un warning
#	Elle vérifie si la commande précédente a fonctionné correctement.
#	Si elle a renctré un erreur, elle affiche un code d'erreur ainsi qu'un message d'erreur. 
#		Dans le cas d'une erreur, elle déclenche un rollback puis arrête le script avec le code d'erreur spécifié.
#		Dans le cas d'un warning, le script continue.
# Paramètres :
#	1 : est une chaine contenant "error" ou "warning" pour déterminer le niveau d'erreur
#	2 : est un code d'erreur
#	3 : est le message d'erreur à afficher
#	4 : est le niveau de rollback à effectuer
#	5 : est vide si l'on veut que la detection de l'erreur se base sur $?
#           est "force" si l'on forcer la détection d'une erreur
#	6 : est le chemin init_dir (pour passer à rollbackInstall ())
# ****************************************
handleError ()
{
	if [ $? -ne 0 ] || [ "$5" == "force" ]; then
		if [ "$1" == "error" ]; then
			echo -e "Erreur (code $2): $3"
			rollbackInstall $4 $6
			exit $2
		elif [ "$1" == "warning" ]; then
			echo -e "Warning (code $2): $3"
		else
			echo "Type d'erreur non supporté"
		fi
	fi
}
rollbackLevel="" # vide par défaut --> rollback n'annule rien
backedUpFiles=""
installFilesPath=`dirname $0`
if [ "`uname -s`" == "VMkernel" ] || [ "`uname -r | grep \".ESX$\"`" != "" ]; then isESX=true; else isESX=false; fi
#### Vérification que l'utilisateur est bien "root" pour ESX
if [ "`uname -r | grep \".ESX$\"`" != "" ] && [ ! $(whoami) == "root" ]; then
	echo "Vous devez être root pour lancer ce script."
	exit 99
fi
#### Vérification que l'utilisateur est bien "root" pour ESXi (on utilise cette méthode car ESXi n'a pas de commande "id" ni "whoami")
if [ "`uname -s`" == "VMkernel" ] && [ ! $(env | grep -e "^USER=" | awk -F = '{print $2}') == "root" ]; then
	echo "Vous devez être root pour lancer ce script."
	exit 99
fi
#### Création du chemin de destination de l'installation
if [ ! -d /usr/local ]; then
	mkdir /usr/local
	handleError "error" 1 "Echec de création du chemin /usr/local" $rollbackLevel
	rollbackLevel="$rollbackLevel"a # supprime /usr/local
fi
if [ ! -d /usr/local/dsr ]; then
	mkdir /usr/local/dsr
	handleError "error" 2 "Echec de création du chemin /usr/local/dsr" $rollbackLevel
	rollbackLevel="$rollbackLevel"b # supprime /usr/local/dsr
fi
#### Détection du chemin des scripts de démarrage
init_dir=/etc/init.d
if [ -d /etc/init.d ]; then
	init_dir="/etc/init.d"
else
	if [ -d /etc/rc.d/init.d ]; then
		init_dir="/etc/rc.d/init.d"
	else
		handleError "error" 3 "Dossier de scripts non détecté" $rollbackLevel "force"
	fi
fi
#### Détection de la commande utilisée pour copier les fichiers
INSTALL="install -c -m 755"
if [ -z "`which install`" ]; then
	INSTALL="cp"
fi
if [ "$isESX" == "true" ]	# ESX (une commande install existe sous ESX mais n'est pas celle que l'on veut utiliser ici)
then
	INSTALL="cp"
fi
#### Arrêt de l'agent s'il est déjà installé
if [ -f "$init_dir/dsr" ]; then
	echo "DSR existe deja, je l'arrete"
	$init_dir/dsr stop
	handleError "error" 7 "Impossible d'arrêter l'ancienne version de DSR." $rollbackLevel $init_dir
	rollbackLevel="$rollbackLevel"f # redémarre l'ancienne version de DSR
fi
#### Copie des fichiers de l'agent
PLATEFORME=`uname -m`
if [ -e /usr/local/dsr/dsr ]; then
	if [ -f /usr/local/dsr/dsr.beemoinstallbak ]; then
			rm /usr/local/dsr/dsr.beemoinstallbak
	fi
	mv /usr/local/dsr/dsr /usr/local/dsr/dsr.beemoinstallbak
	backedUpFiles="$backedUpFiles "/usr/local/dsr/dsr.beemoinstallbak
fi
if [ "$PLATEFORME" = "ia64" ]; then
	$INSTALL $installFilesPath/dsr_ia64 /usr/local/dsr/dsr
	handleError "error" 13 "Echec d'installation du fichier /usr/local/dsr/dsr" $rollbackLevel
	rollbackLevel="$rollbackLevel"c # supprime /usr/local/dsr/dsr
else
	if [ "$PLATEFORME" = "x86_64" ]; then
		if [ "$isESX" == "false" ]; then
			$INSTALL $installFilesPath/dsr_x64 /usr/local/dsr/dsr
			handleError "error" 13 "Echec d'installation du fichier /usr/local/dsr/dsr" $rollbackLevel
			rollbackLevel="$rollbackLevel"c # supprime /usr/local/dsr/dsr
		else
			$INSTALL $installFilesPath/dsr_x86 /usr/local/dsr/dsr
			handleError "error" 13 "Echec d'installation du fichier /usr/local/dsr/dsr" $rollbackLevel
			rollbackLevel="$rollbackLevel"c # supprime /usr/local/dsr/dsr
			chmod +x /usr/local/dsr/dsr
			if [ ! -d /usr/local/dsr/scripts ]; then
				mkdir /usr/local/dsr/scripts
				handleError "error" 4 "Echec de création du chemin /usr/local/dsr/scripts" $rollbackLevel
				rollbackLevel="$rollbackLevel"d # supprime /usr/local/dsr/scripts
			fi
			if [ -e /usr/local/dsr/scripts/backup_config.sh ]; then
				if [ -f /usr/local/dsr/scripts/backup_config.sh.beemoinstallbak ]; then
					rm /usr/local/dsr/scripts/backup_config.sh.beemoinstallbak
				fi
				cp /usr/local/dsr/scripts/backup_config.sh /usr/local/dsr/scripts/backup_config.sh.beemoinstallbak
				backedUpFiles="$backedUpFiles "/usr/local/dsr/scripts/backup_config.sh.beemoinstallbak
			fi
			if [ -d "/scratch" ] && [ ! -e "/usr/local/dsr/conf/logs" ]; then
				mkdir -p /scratch/dsr_logs
				mkdir -p /usr/local/dsr/conf
			fi
			if [ -d "/var/log" ] && [ ! -e "/usr/local/dsr/conf/logs" ]; then
				mkdir -p /var/log/dsr_logs
				mkdir -p /usr/local/dsr/conf
			fi
			$INSTALL $installFilesPath/backup_config.sh /usr/local/dsr/scripts
			handleError "error" 5 "Echec d'installation du fichier /usr/local/dsr/scripts/backup_config.sh" $rollbackLevel
			rollbackLevel="$rollbackLevel"e # supprime /usr/local/dsr/scripts/backup_config.sh
			chmod +x /usr/local/dsr/scripts/backup_config.sh
			handleError "error" 6 "Echec d'ajout du droit d'exécution au fichier /usr/local/dsr/scripts/backup_config.sh" $rollbackLevel
			## Authorisation du bind dans ESXi 5.0+
			if [ -d "/etc/vmware/secpolicy/tardisks/" ]; then
				echo "-E unlabeled superDom" > /etc/vmware/secpolicy/tardisks/dsr
			fi



		fi
	else
		$INSTALL $installFilesPath/dsr_x86 /usr/local/dsr/dsr
		handleError "error" 13 "Echec d'installation du fichier /usr/local/dsr/dsr" $rollbackLevel
		rollbackLevel="$rollbackLevel"c # supprime /usr/local/dsr/dsr
	fi
fi
#### Installation du script de démarrage de l'agent
if [ -e $init_dir/dsr ]; then
	if [ -f $init_dir/dsr.beemoinstallbak ]; then
		rm $init_dir/dsr.beemoinstallbak
	fi
	cp $init_dir/dsr $init_dir/dsr.beemoinstallbak
	backedUpFiles="$backedUpFiles "$init_dir/dsr.beemoinstallbak
fi
if [ -f $init_dir/dsr ]; then
	rm $init_dir/dsr
fi
cp $installFilesPath/init $init_dir/dsr
handleError "error" 8 "Echec d'installation du fichier $init_dir/dsr" $rollbackLevel $init_dir
rollbackLevel="$rollbackLevel"g # supprime $init_dir/dsr
chmod +x $init_dir/dsr
handleError "error" 9 "Echec d'ajout du droit d'exécution au fichier $init_dir/dsr" $rollbackLevel "" $init_dir
if [ -x /sbin/chkconfig ]		# RedHat + Mandrake + ESX 5.1 et plus anciens
then
	if [ ! "`/sbin/chkconfig --list | grep "^dsr "`" ]; then
		/sbin/chkconfig --add dsr
		handleError "warning" 10 "\nLes Scripts de démarrage n'ont pas été créé automatiquement\nPour que Data Safe Restore se lance au démarrage de votre machine\najoutez $init_dir/dsr start a vos scripts de démarrage" $rollbackLevel "" $init_dir
		rollbackLevel="$rollbackLevel"h # enlève le script de démarrage DSR des scripts à lancer automatiquement (chkconfig)
	fi
	if [ "`/sbin/chkconfig --list | grep "^dsr " | awk -F" +" '{print $2}'`" == "off" ]; then
		/sbin/chkconfig dsr on
		handleError "warning" 10 "\nLes Scripts de démarrage n'ont pas été créé automatiquement\nPour que Data Safe Restore se lance au démarrage de votre machine\najoutez $init_dir/dsr start a vos scripts de démarrage" $rollbackLevel "" $init_dir
		rollbackLevel="$rollbackLevel"h # enlève le script de démarrage DSR des scripts à lancer automatiquement (chkconfig)
	fi
elif [ -x /bin/chkconfig ]		# ESX 5.5
then
	if [ ! "`/bin/chkconfig --list | grep "^dsr "`" ]; then
		/bin/chkconfig --add dsr
		handleError "warning" 10 "\nLes Scripts de démarrage n'ont pas été créé automatiquement\nPour que Data Safe Restore se lance au démarrage de votre machine\najoutez $init_dir/dsr start a vos scripts de démarrage" $rollbackLevel "" $init_dir
		rollbackLevel="$rollbackLevel"h # enlève le script de démarrage DSR des scripts à lancer automatiquement (chkconfig)
	fi
	if [ "`/bin/chkconfig --list | grep "^dsr " | awk -F" +" '{print $2}'`" == "off" ]; then
		/bin/chkconfig dsr on
		handleError "warning" 10 "\nLes Scripts de démarrage n'ont pas été créé automatiquement\nPour que Data Safe Restore se lance au démarrage de votre machine\najoutez $init_dir/dsr start a vos scripts de démarrage" $rollbackLevel "" $init_dir
		rollbackLevel="$rollbackLevel"h # enlève le script de démarrage DSR des scripts à lancer automatiquement (chkconfig)
	fi
elif [ -x /sbin/rc-update ]		# Gentoo
then
##	if [ ! "`/sbin/rc-update show default | grep "^dsr "`" ]; then # vérifier la sortie de la commande pour voir la les lignes commencent par <nom du scrpit> suivi d'un espace
		/sbin/rc-update add dsr default
		handleError "warning" 10 "\nLes Scripts de démarrage n'ont pas été créé automatiquement\nPour que Data Safe Restore se lance au démarrage de votre machine\najoutez $init_dir/dsr start a vos scripts de démarrage" $rollbackLevel "" $init_dir
##		rollbackLevel="$rollbackLevel"i # enlève le script de démarrage DSR des scripts à lancer automatiquement (rc-update)
##	fi
elif [ -x /usr/sbin/update-rc.d ]	# Debian
then
##	if [ ! `` ]; then # update-rc.d n'a pas de fonction de listage
		/usr/sbin/update-rc.d -f dsr defaults
		handleError "warning" 10 "\nLes Scripts de démarrage n'ont pas été créé automatiquement\nPour que Data Safe Restore se lance au démarrage de votre machine\najoutez $init_dir/dsr start a vos scripts de démarrage" $rollbackLevel "" $init_dir
##		rollbackLevel="$rollbackLevel"j # enlève le script de démarrage DSR des scripts à lancer automatiquement (update-rc.d)
##	fi
else
	echo "Les Scripts de démarrage n'ont pas été créé automatiquement"
	echo "Pour que Data Safe Restore se lance au démarrage de votre machine"
	echo "ajoutez $init_dir/dsr start a vos scripts de démarrage"
fi
#### Pour ESXi backup_config.sh rend l'installation permanente (permet de survivre au reboot du serveur)
if [ "`uname -s`" == "VMkernel" ]
then
	/usr/local/dsr/scripts/backup_config.sh
	handleError "error" 11 "Echec lors de l'exécution /usr/local/dsr/scripts/backup_config.sh" $rollbackLevel "" $init_dir
fi
if [ "$backedUpFiles" ]; then
	rm $backedUpFiles
fi
$init_dir/dsr start
handleError "error" 12 "Installation Data Safe Restore Client - OK, mais impossible de démarrer l'agent" ""
echo "Installation Data Safe Restore Client - OK"
