#!/bin/sh

#################################################################
# nom			: uninstall.sh
# description	: Scrit de désinstallation de l'agent DSR sous systèmes de type unix
# version		: 1.10
# date			: 09/12/2013 16:00
# autheur		: remi & flo
#################################################################
# description: Client Data Safe Restore uninstall script
# 1.1 24/06/2022 - support ESXi 7.0 

LOCAL_PORT=6666
REMOTE_PORT=6666
DSRDIR=/usr/local/dsr
CONFIGFILE="$DSRDIR/conf/config"

if [ "`uname -s`" == "VMkernel" ] || [ "`uname -r | grep \".ESX$\"`" != "" ]; then isESX=true; else isESX=false; fi

if [ -x /sbin/chkconfig ]		# RedHat + Mandrake + ESX
then
	/sbin/chkconfig --del dsr
elif [ -x /sbin/rc-update ]		# Gentoo
then
	/sbin/rc-update del dsr
elif [ -x /usr/sbin/update-rc.d ]	# Debian
then
	/usr/sbin/update-rc.d -f dsr remove
else
	echo "Les Scripts de démarrage n'ont pas été supprimé automatiquement"
	echo "Pour que Data Safe Restore ne se lance plus au démarrage de votre machine"
	echo "supprimez /etc/init.d/dsr start de vos scripts de démarrage"
fi

/etc/init.d/dsr stop

rm /etc/init.d/dsr

if [ -f "$CONFIGFILE" ]; then
	PORT=`grep "^local_port=" $CONFIGFILE | awk -F "=" '{ print $2 }'`
	if [ ! -z "$PORT" ] && [ "$PORT" -eq "$PORT" 2> /dev/null ]; then LOCAL_PORT=$PORT; fi
	PORT=`grep "^server_port=" $CONFIGFILE | awk -F "=" '{ print $2 }'`
	if [ ! -z "$PORT" ] && [ "$PORT" -eq "$PORT" 2> /dev/null ]; then REMOTE_PORT=$PORT; fi
fi

if [ -d /usr/local/dsr ]; then
	rm -Rf /usr/local/dsr > /dev/null 2> /dev/null &
	if [ $? -ne 0 ]; then
		echo -e "\nLa supression de certains fichiers a échoué."
		if [ "`uname -s`" == "VMkernel" ]	# ESX
		then
			echo -e " Ces fichiers seront supprimés lors du prochain redémarrage de serveur ESX."
		else
			echo -e " Merci de supprimer /usr/local/dsr manuellement."
		fi
	fi
fi

if [ "$isESX" == "true" ]; then	# ESX
	# suppression du service beemo dans le firewall
	if  [ -x /sbin/esxcli ] && [ -e /etc/vmware/firewall/service.xml ]; then
		if [ -f /tmp/service.xml.beemoremovebak ]; then
			rm /tmp/service.xml.beemoremovebak
		fi
		if [ "`uname -s`" == "VMkernel" ] && [ `uname -r | awk -F. '{print $1}'` -ge 7 ]; then # ESXI 7+
	  	    if [ -f /etc/vmware/firewall/beemo.xml ]; then
		  	    echo -e "\nremove /etc/vmware/firewall/beemo.xml ESXI 7.+ detecté"	
		  	    rm /etc/vmware/firewall/beemo.xml
		  	    sleep 10
				esxcli network firewall refresh
	  	    fi  
	  	else # ESXI 5/6
	  	    if [ -f /etc/vmware/firewall/service.xml.beemobak ]; then #remove old config
			    cp /etc/vmware/firewall/service.xml /tmp/service.xml.beemoremovebak
			    sed 's#<!-- Beemo Technologie DataSafeRestore Agent -->.*<!-- Beemo Technologie DataSafeRestore Agent -->##' -i /etc/vmware/firewall/service.xml
			    if [ $? -ne 0 ]; then
				    mv /tmp/service.xml.beemoremovebak /etc/vmware/firewall/service.xml
				    echo -e "\nLa supression du service de firewall DSR a échoué. Le supprimer manuellement, sinon il sera supprimé au prochain reboot du serveur."
			    else
	  	            rm /etc/vmware/firewall/service.xml.beemobak
	  	            echo -e "\nremove firewall config in /etc/vmware/firewall/service.xml ESXI 5/6 detecté"	
				    sleep 10
				    esxcli network firewall refresh
			    fi
			fi
			if [ -f /etc/vmware/firewall/beemo.xml ]; then # remove new config
		  	    echo -e "\nremove /etc/vmware/firewall/beemo.xml ESXI 5/6 detecté"	
		  	    rm /etc/vmware/firewall/beemo.xml
		  	    sleep 10
				esxcli network firewall refresh
	  	    fi 
	    fi
	elif [ -x /sbin/esxcfg-firewall ] || [ -x /usr/sbin/esxcfg-firewall ]; then
		esxcfg-firewall --closePort $REMOTE_PORT,tcp,out,DSR
		esxcfg-firewall --closePort $LOCAL_PORT,tcp,in,DSR
		# service firewall restart
	fi

	# suppression du module de boot beemo
	if [ -f /bootbank/boot.cfg.beemoremovebak ]; then
			rm /bootbank/boot.cfg.beemoremovebak
	fi
	if [ "`uname -s`" == "VMkernel" ]; then # Pas de /bootbank dans ESX
		cp /bootbank/boot.cfg /bootbank/boot.cfg.beemoremovebak
		sed -e '/modules=/s/ --- beemo.tgz//' -i /bootbank/boot.cfg
		if [ $? -ne 0 ]; then
			mv /bootbank/boot.cfg.beemoremovebak /bootbank/boot.cfg
			echo -e "\nLa supression du module de boot beemo.tgz a échoué. La désinstallation a échoué."
			exit 1
		fi
		rm /bootbank/beemo.tgz
	fi
	
	# suppression des logs
	if [ -d "/scratch/dsr_logs" ]; then
		rm -Rf /scratch/dsr_logs > /dev/null 2> /dev/null &
	fi
	if [ -d "/var/log/dsr_logs" ]; then
		rm -Rf /var/log/dsr_logs > /dev/null 2> /dev/null &
	fi
	
	# suppression du fichier d'authorisation du bind dans ESXi 5.0+
	if [ -f /etc/vmware/secpolicy/tardisks/dsr ]; then
		rm /etc/vmware/secpolicy/tardisks/dsr
	fi
	
	# suppression des fichiers de backup de boot.cfg
	if [ -f /bootbank/boot.cfg.beemoinstallbak ]; then
			rm /bootbank/boot.cfg.beemoinstallbak
	fi
	if [ -f /bootbank/boot.cfg.beemoremovebak ]; then
			rm /bootbank/boot.cfg.beemoremovebak
	fi
fi

echo "Desinstallation Data Safe Restore Client terminée"
